generateSNP<-function(N,P){
	num<-rmultinom(1,N, c((1-P)*(1-P), (1-P)*P*2, P*P))[,1];
	res<-sample(c(rep(0,num[1]),rep(1,num[2]),rep(2,num[3])));
	return(res);
}
generateSet<-function(D){
	pathSets<-list();
	for(i in 2:D){
		subPaths <- apply(expand.grid(rep(list(c(FALSE,TRUE)), i))[-1,],1,function(t) as.numeric(which(t)));
		pathSets[[i]]<-subPaths;
	}
	return(pathSets);
}
extractFeature<-function(x ,y, pathStats, prob)
{
	stats<-data.frame(name="fake",importance=numeric(1))
	stats[1,]$importance <- 100;
	data<-cbind(x,y);
	for(i in 1:length(names(x))){
		p <- getChiSquareP(data, names(x)[i]);
		if(p < prob){
			oneStat<-data.frame(name="fake",importance=numeric(1));
			oneStat[1]<-names(x)[i];
			oneStat[2]<-p;
			stats<-rbind(stats,oneStat);
		}
	}
	for(i in 1:nrow(pathStats)){
		strs<-unlist(strsplit(as.character(pathStats[i,]$name), " "));
		if(length(strs) > 2){
			for(r in 1:length(strs)){
				if(! strs[r] %in% stats$name){
					oneStat<-data.frame(name="fake",importance=numeric(1));
					oneStat[1]<-strs[r];
					oneStat[2]<-getChiSquareP(data, strs[r]);
					stats<-rbind(stats,oneStat);
				}
			}
		}
		else{
			p<-getPathChiSquareP(data, strs[1], strs[2]);
			if(p < prob){
				print(paste("Interaction: ", strs[1], strs[2], p, sep = " "));
				for(r in 1:length(strs)){
					if(! strs[r] %in% stats$name){
						oneStat<-data.frame(name="fake",importance=numeric(1));
						oneStat[1]<-strs[r];
						oneStat[2]<-getChiSquareP(data, strs[r]);
						stats<-rbind(stats,oneStat);
					}
				}
			}
		}
	}
	stats<-stats[stats$importance<1,];
	stats<-stats[order(stats$importance,decreasing=FALSE),];
	return(stats);
}
testFeature<-function(x, gbmObjectList)
{
	RATIO<-2;
	rnames<-names(x);
	stats<- data.frame(name=rnames, importance=numeric(length(rnames)),count=numeric(length(rnames)));
	for(i in 1:length(gbmObjectList)){
		gbmObject<-gbmObjectList[[i]];
		features<- summary(gbmObject,plotit = FALSE)
		features <- features[features$rel.inf>0,];
		for(row in 1:nrow(features)){
			featureName<-as.character(features[row,]$var);
			importance<-features[row,]$rel.inf;
			stats[stats$name== featureName,"importance"] <- stats[stats$name==featureName,"importance"] + importance;		
			stats[stats$name== featureName,"count"] <- stats[stats$name==featureName,"count"] + 1;
		}
	}
	stats<-stats[stats$importance>0,];
	stats<-stats[order(stats$importance+stats$count*20000,decreasing=TRUE),];
	if(nrow(stats) > length(rnames)/RATIO)
	{
		stats<-stats[1:(length(rnames)/RATIO),];
	}
	return(stats);

}
extractAccuracy<-function(gbmObjectList,x,y,randcv)
{
	N<-nrow(x);
	err.cv <- rep(N/length(gbmObjectList),length(gbmObjectList));
	for(i in 1:length(gbmObjectList)){
		gbmObject<-gbmObjectList[[i]];
		f.predict <- predict.gbm(gbmObject,x[randcv==i,],n.trees=gbmObject$n.tree);
		p.pred <- 1/(1+exp(-f.predict));
		err.cv[i] <- sum(abs((sign(p.pred-0.5)+1)/2-y[randcv==i]));
	}

print(err.cv);
	return((N-sum(err.cv))/N);
}
extractPath<-function(treeObject,DEPTH)
{
	res<-list();
	index<-1;
	paths<-list();
	paths[[1]]<-c(0);
	for(d in 1:DEPTH)
	{
		tmpPaths<-list();
		tmpIndex<-1;
		if(length(paths) == 0) break;
		for(i in 1:length(paths))
		{
			path<-paths[[i]];
			lastIndex<-length(path);
			#nodeIndex + 1 == rowIndex
			lastRow<-treeObject[(path[lastIndex]+1),];
			leftRow<-lastRow$LeftNode;
			rightRow<-lastRow$RightNode;
			
			if(lastRow$SplitVar != -1)
			{
				if((lastRow$SplitVar+1) %in% path[1:(lastIndex-1)])
				{
					path[lastIndex]<-leftRow;
					tmpPaths[[tmpIndex]]<- path;
					tmpIndex<-tmpIndex+1;
					path[lastIndex]<-rightRow;
					tmpPaths[[tmpIndex]]<- path;
					tmpIndex<-tmpIndex+1;

				}else{
					path[lastIndex]<-lastRow$SplitVar+1;
					if(length(path)>1)
					{
						res[[index]]<-path;
						index<-index+1;
					}	
					tmpPaths[[tmpIndex]]<-c(path,leftRow);
					tmpIndex<-tmpIndex+1;
					tmpPaths[[tmpIndex]]<-c(path,rightRow);
					tmpIndex<-tmpIndex+1;
				}
			}
		}
		paths<-tmpPaths;
	}
	return(res);
}
extractInteract<-function(gbmObject, x, numTrees, pathSets,DEPTH)
{
	pathStats<-data.frame(name="fake",importance=numeric(1))
	for(j in 1:numTrees)
	{
		#print(paste("tree",j,sep=""));
		a<-pretty.gbm.tree(gbmObject,i.tree = j);
		pathSet<-extractPath(a, DEPTH);
		if(length(pathSet) < 1)
		{
			print("Error");
			print(a);
		}else
		{
		for(i in 1:length(pathSet))
		{
			p<-pathSet[[i]];
			p<-sort(p);
			tmpStr<-p[1];
			for(l in 2:length(p))
			{
				tmpStr<-paste(tmpStr, p[l], sep=",");
			}
			if(! tmpStr %in% pathStats$name)
			{
				inter = interact.gbm(gbmObject,x,i.var=p, n.trees=numTrees)
				#inter = runif(1);
				#print("interaction");
				#print(p);
				#print(inter);
				#if(inter > (1/length(p))){
				if(inter > 0){
					onePath<-data.frame(name="fake",importance=numeric(1));
					onePath[1]<-tmpStr;
					onePath[2]<-inter;
					pathStats<-rbind(pathStats,onePath);
				}
			 }
		}
		}
	}
	pathStats<-pathStats[pathStats$importance>0,];
	returnStats<-data.frame(name="fake",importance=numeric(1));
	if(nrow(pathStats) > 0)
	{
	for(s in 1:nrow(pathStats)){
		onePath<-as.character(pathStats[s,]$name);
		strs<-unlist(strsplit(onePath, ","));
		refs<-c();
		for(r in 1:length(strs)){
			refs<-c(refs, gbmObject$var.names[as.numeric(strs[r])]);
		}
		tmpStr<-refs[1];
		for(l in 2:length(refs)){
	 		tmpStr<-paste(tmpStr, refs[l], sep=" ");
		}
		oneStats<-data.frame(name="fake",importance=numeric(1))
		oneStats[1]<-tmpStr;
		oneStats[2]<-pathStats[s,]$importance;
		returnStats<-rbind(returnStats,oneStats);
	}
	}
	#print(returnStats);
	returnStats<-returnStats[returnStats$importance>0,];
	return(returnStats);
}

extractInteractStats<-function(gbmObjectList, x, numTrees,pathSets,DEPTH)
{
	pathStats<-data.frame(name="fake",importance=numeric(1),count=numeric(1));
	for(i in 1:length(gbmObjectList)){
		print(paste("Iteration ",i));
		gbmObject<-gbmObjectList[[i]];
		onePathStats<-extractInteract(gbmObject,x,numTrees,pathSets,DEPTH);
		if(nrow(onePathStats)>0)
		{
		for(i in 1:nrow(onePathStats))
		{
			pathName<-as.character(onePathStats[i,]$name);
			importance<-onePathStats[i,]$importance;
			if(pathName %in% pathStats$name)
			{	
				pathStats[pathStats$name== pathName,"importance"] <- pathStats[pathStats$name==pathName,"importance"] + importance;		
				pathStats[pathStats$name== pathName,"count"] <- pathStats[pathStats$name==pathName,"count"] + 1;
			}
			else
			{
				tmpStats<-data.frame(name="fake",importance=numeric(1),count=numeric(1));
				tmpStats[1]<-pathName;
				tmpStats[2]<-importance;
				tmpStats[3]<-1;
				pathStats<-rbind(pathStats,tmpStats);
			}

		}
		}
	}
	pathStats<-pathStats[order(pathStats$importance+pathStats$count*20000,decreasing=TRUE),];

	tmpPathStats<-data.frame(name="fake",importance=numeric(1),count=numeric(1));
	counter<-rep(0,DEPTH);
	for(s in 1:nrow(pathStats)){
		onePath<-as.character(pathStats[s,]$name);
		strs<-unlist(strsplit(onePath, ","));
		len<-length(strs);
		if(counter[len] < 100)
		{
			tmpPathStats<-rbind(tmpPathStats,pathStats[s,]);
			counter[len]<-counter[len]+1;
		}
	}

	pathStats<-tmpPathStats;
	pathStats<-pathStats[pathStats$importance>0,];
	return(pathStats);
}
snpFinder<-function(loadeddata,WINDOW,OVERLAP,NTREE,randcv,DEPTH)
{
	data <- loadeddata[,1:ncol(loadeddata)-1];
	y <- loadeddata[,ncol(loadeddata)];
	while(TRUE){
		START<-1;
		TOTAL<-ncol(data);
		featureSet<-NULL;
		while(TRUE){
			END<-START+WINDOW-1;
			if(END > TOTAL){
				END<-TOTAL;
			}
			if(TOTAL - END < WINDOW/4)
			{
				END<-TOTAL;
			}		
			x<-data[,START:END];
			gbmObjectList<-list();
			for (i in sort(unique(randcv)))
			{
				gbmObject<- gbm.fit(x[randcv!=i,],y[randcv!=i],
		               	offset = NULL,
            		   	misc = NULL,
		               	distribution = "bernoulli",#bernoulli
            		   	w = NULL,
		               	var.monotone = NULL,
		               	n.trees = NTREE,
		               	interaction.depth = DEPTH,
		               	n.minobsinnode = 10,
		               	shrinkage = .1,
		               	bag.fraction = 0.5,
		               	train.fraction =1,
		               	keep.data = TRUE,
		               	verbose = FALSE,
		               	var.names = NULL,
		               	response.name = NULL);
				gbmObjectList[[i]]<-gbmObject;
			}

			accuracy<-extractAccuracy(gbmObjectList,x,y,randcv);
			selectedFeature<-testFeature(x,gbmObjectList);
			for(i in 1:nrow(selectedFeature))
			{
				if(!(as.character(selectedFeature[i,]$name) %in% featureSet$name))
					featureSet<-rbind(featureSet,selectedFeature[i,]);
			}
			START<-START+WINDOW-OVERLAP;
			if(END == TOTAL) break;
		}
		data<-loadeddata[as.character(featureSet$name)];
		if(nrow(featureSet) <= WINDOW)
			break
	}
	return(featureSet);
}
getDataFromSeparation<-function(controlFolder, controlName, caseFolder, caseName, chr, SEPERATION, NTREE,DEPTH,MISSINGFILTER)
{
	mergedData<-data.frame(); 
	mergedY<-NULL;
	for(sep in 1:SEPERATION)
	{
		controlFileName<-paste(controlFolder,controlName,chr,"_",iteration,".dat",sep="");
		caseFileName<-paste(caseFolder,caseName,chr,"_",iteration,".dat",sep="");
		if(!file.exists(controlFileName) || !file.exists(caseFileName)){
			break;
		}
		controlData <- read.table(controlFileName,head = T,sep = "\t");
		caseData <- read.table(caseFileName,head = T,sep = "\t");
		loadeddata<-rbind(controlData,caseData);
		x <- loadeddata[,-ncol(loadeddata)];
		y <- loadeddata[,ncol(loadeddata)];
		missval<-NULL;
		for(i in 1:ncol(x)){ 
			if(sum(is.na(x[,i]))> MISSINGFILTER){
				missval <- rbind(missval,i)
			}
		}
		if(length(missval) > 0)
			x <- x[,-missval];
		for(i in 1:ncol(x)){ 
			x[,i] <- factor(x[,i])
		}

		N <- nrow(x)
		gbmObject<- gbm.fit(x,y,
               		offset = NULL,
	               	misc = NULL,
      	         	distribution = "bernoulli",#bernoulli
            	   	w = NULL,
               		var.monotone = NULL,
	               	n.trees = NTREE,
      	         	interaction.depth = DEPTH,
            	   	n.minobsinnode = 10,
	               	shrinkage = .1,
      	         	bag.fraction = 0.5,
            	   	train.fraction =1,
	               	keep.data = TRUE,
	               	verbose = FALSE,
	               	var.names = NULL,
	               	response.name = NULL);
		features<- summary(gbmObject,plotit = FALSE)
		features <- features[features$rel.inf>0,];
		if(nrow(features) > ncol(x)/3)
		{
			features<-features[1:(ncol(x)/3),];
		}
		if(ncol(mergedData) == 0)
		{
			mergedData<-x[as.character(features$name)];
			mergedY<-y;
		}else
		{
			mergedData<-cbind(mergedData,x[as.character(features$name)]);
		}
	}
	return(cbind(mergedData,mergedY));
}

tmp<-function()
{
	x<-data;
	gbmObjectList<-list();
	for (i in sort(unique(randcv)))
	{
		gbmObject<- gbm.fit(x[randcv!=i,],y[randcv!=i],
               	offset = NULL,
               	misc = NULL,
               	distribution = "bernoulli",#bernoulli
               	w = weight,
               	var.monotone = NULL,
               	n.trees = ntree,
               	interaction.depth = depth,
               	n.minobsinnode = 10,
               	shrinkage = .1,
               	bag.fraction = 0.5,
               	train.fraction =1,
               	keep.data = TRUE,
               	verbose = FALSE,
               	var.names = NULL,
               	response.name = NULL);
		gbmObjectList[[i]]<-gbmObject;
	}

	#selectedFeature<-testFeature(x,gbmObjectList);
	accuracy<-extractAccuracy(gbmObjectList,x,y,randcv);
	pathSets<-generateSet(DEPTH);
	for(i in 1:length(pathSets))
	{
		pathSet<-pathSets[[i]];
		tmpSet<-list();
		index<-1;
		for(j in 1:length(pathSet))
		{
			if(length(pathSet[[j]]) > 1)
			{
				tmpSet[[index]]<-pathSet[[j]];
				index<-index+1;	
			}
		}
		pathSets[[i]]<-tmpSet;
	}
	pathStats<-extractInteractStats(gbmObjectList,x,NTREE,pathSets,DEPTH);
	return(pathStats);
}
generateTestData<-function()
{
	for(i in 0:(100-1))
	{
		fileName<-paste(WORKINGFOLDER,"test/dataset/400/25/25.400.",i,".txt",sep="");
		savefileName<-paste(WORKINGFOLDER,"test/newData/25/25.400.",i,".txt",sep="");
		POS<-sort(ceiling(runif(2)*1000));
		fileName<-paste(WORKINGFOLDER,"test/dataset/400/25/25.400.",i,".txt",sep="");
		loadeddata <- read.table(fileName,head = T,sep = "\t");
		for(col in 3:(ncol(loadeddata)-1))
		{
			loadeddata[col]<-generateSNP(nrow(loadeddata),0.20);
		}
		loadeddata<-loadeddata[, c(POS[1],POS[2],3:(POS[1]-1), 1, (POS[1]+1):(POS[2]-1),2, (POS[2]+1):ncol(loadeddata))];
		write.table(loadeddata,file=savefileName,row.names=FALSE,col.names=TRUE,sep="\t");
	}	
}
getDiabetes<-function(iteration)
{
	fileName = paste(WORKINGFOLDER,"data1/parallele_phase_1/cases_chr",iteration,".gty",sep="");
	Case <- read.table(fileName,head = T,sep ='\t');
	fileName = paste(WORKINGFOLDER,"data1/parallele_phase_1/controls_chr",iteration,".gty",sep="");
	print(fileName);
	Ctrl <- read.table(fileName,head = T,sep ='\t');
	Case<-Case[,3:ncol(Case)];
	Ctrl<-Ctrl[,3:ncol(Ctrl)];
	x <- rbind(Case,Ctrl);
	y <- rep(0,nrow(x));
	y[1:816] <- 1;
	loadeddata<-cbind(x,y);
	return(loadeddata);
}
getChiSquareP<-function(datanew, name)
{
	pos = which(names(datanew) == name);
	control<-datanew[datanew$y == 0,pos];
	case<-datanew[datanew$y == 1,pos];
	if(length(which(is.na(control))) >0)
		control<-control[-which(is.na(control))];
	if(length(which(is.na(case))) >0)
		case<-case[-which(is.na(case))];
	dataT = matrix(c(sum(case == 1),  sum(control==1), sum(case==2),sum(control==2),sum(case==3), sum(control==3)),nrow=2);
	dataT<-dataT+1;
	res<-chisq.test(dataT);
	print(dataT);
	return(res$p.value)
}
getAlleleChiSquareP<-function(datanew, name)
{
	res = 10;
	pos = which(names(datanew) == name);
	if(length(pos) == 0){
		return(-1);
	}
	control<-datanew[datanew$y == 0,pos];
	case<-datanew[datanew$y == 1,pos];
	if(length(which(is.na(control))) >0)
		control<-control[-which(is.na(control))];
	if(length(which(is.na(case))) >0)
		case<-case[-which(is.na(case))];
	dataT = matrix(c(sum(case == 1)*2+sum(case==2),  sum(control==1)*2+sum(control==2),sum(case==3)*2+sum(case==2), sum(control==3)*2+sum(control==2)),nrow=2);
	res<-chisq.test(dataT);
	print(dataT);
	return(res$p.value)
}

getPathChiSquareP<-function(datanew, name1, name2)
{
	pos1 = which(names(datanew) == name1);
	pos2 = which(names(datanew) == name2);
	if(length(pos1) == 0){
		return(100);
	}
	if(length(pos2) == 0){
		return(100);
	}
	control <-datanew[datanew$y == 0,c(pos1,pos2)];
	case  <-datanew[datanew$y == 1,c(pos1,pos2)];
	control <-na.omit(control);
	case <-na.omit(case);
	control<-(as.numeric(control[,1])-1)*10+(as.numeric(control[,2])-1);
	case<-(as.numeric(case[,1])-1)*10+(as.numeric(case[,2])-1);
	dataT = matrix(c(sum(case == 0),  sum(control==0), sum(case==1),sum(control==1),sum(case==2), sum(control==2),sum(case == 10),  sum(control==10), sum(case==11),sum(control==11),sum(case==12), sum(control==12),sum(case == 20),  sum(control==20), sum(case==21),sum(control==21),sum(case==22), sum(control==22)),nrow=2);
	dataT<-dataT+1;
	res<-chisq.test(dataT);
	if(res$p.value < 0.00001)
		print(dataT);
	return(res$p.value)
}

